@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 from __future__ import absolute_import
 
+import logging
 import os
 import shutil
 import sys
@@ -65,16 +66,56 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         # We do this so that the window is only hidden when the
         # user click on the close button provided by the window manager.
         self.connect('delete-event', self.on_destroy)#lambda w, e: True)
-        self.set_default_size(400, 400)
-        self.g_notebook = gtk.Notebook()
-        self.vbox.pack_start(self.g_notebook)
 
-        ########
-        # midi #
-        ########
-        page_vbox = gu.hig_dlg_vbox()
-        self.g_notebook.append_page(page_vbox, gtk.Label(_("Instruments")))
+        hbox = gtk.HBox()
+        hbox.set_spacing(gu.hig.SPACE_LARGE)
+        hbox.set_border_width(gu.hig.SPACE_SMALL)
+        self.vbox.pack_start(hbox, True)
 
+        frame = gtk.Frame()
+        self.g_pages = gtk.TreeStore(str)
+        self.g_pview = gtk.TreeView(self.g_pages)
+        self.g_pview.set_headers_visible(False)
+        hbox.pack_start(frame, False)
+        frame.add(self.g_pview)
+
+        self.g_page_box = gtk.HBox()
+        hbox.pack_start(self.g_page_box, True)
+        self.m_page_mapping = {}
+
+        def cursor_changed(treeview):
+            path, col = treeview.get_cursor()
+            for key, page in self.m_page_mapping.items():
+                if key == path:
+                    page.show()
+                else:
+                    page.hide()
+            self.m_page_mapping[path].show_all()
+        tvcol = gtk.TreeViewColumn("Col 0")
+        self.g_pview.append_column(tvcol)
+        cell = gtk.CellRendererText()
+        tvcol.pack_start(cell, True)
+        tvcol.add_attribute(cell, 'text', 0)
+        hbox.show_all()
+
+        self.create_midi_config()
+        self.create_user_config()
+        self.create_external_programs_config()
+        self.create_gui_config()
+        self.create_practise_config()
+        self.create_sound_config()
+        self.create_statistics_config()
+        self.g_pview.connect('cursor-changed', cursor_changed)
+        self.g_pview.set_cursor((0, ), )
+    def new_page_box(self, parent, heading):
+        page_vbox = gtk.VBox()
+        page_vbox.set_spacing(gu.hig.SPACE_MEDIUM)
+        self.g_page_box.pack_start(page_vbox)
+        it = self.g_pages.append(parent, [heading])
+        self.m_page_mapping[self.g_pages.get_path(it)] = page_vbox
+        return it, page_vbox
+    def create_midi_config(self):
+        it, page_vbox = self.new_page_box(None, _("Instruments"))
         vbox, category_vbox = gu.hig_category_vbox(_("Tempo"))
         page_vbox.pack_start(vbox, False)
         sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
@@ -121,12 +162,8 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             _("Rhythm:"),
             gu.PercussionNameButton("config", "rhythm_perc", "Side Stick"),
             sizegroup, True, True))
-        ########
-        # user #
-        ########
-        page_vbox = gu.hig_dlg_vbox()
-        self.g_notebook.append_page(page_vbox, gtk.Label(_("User")))
-
+    def create_user_config(self):
+        it, page_vbox = self.new_page_box(None, _("User"))
         box, category_vbox = gu.hig_category_vbox(_("User's Vocal Range"))
         page_vbox.pack_start(box, False)
         sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
@@ -160,12 +197,8 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         category_vbox.pack_start(self.g_sex_female, False)
         if self.get_string('user/sex') == 'female':
             self.g_sex_female.set_active(True)
-        #####################
-        # external programs #
-        #####################
-        page_vbox = gu.hig_dlg_vbox()
-        self.g_notebook.append_page(page_vbox, gtk.Label(_("External Programs")))
-
+    def create_external_programs_config(self):
+        it, page_vbox = self.new_page_box(None, _("External Programs"))
         box, category_vbox = gu.hig_category_vbox(_("Converters"))
         page_vbox.pack_start(box, False)
         sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
@@ -253,7 +286,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         page_vbox.pack_start(box, False)
 
         # CSound_("Musical MIDI Accompaniment:")
-        for bin, label, bins in (
+        for binary, label, bins in (
             ("csound", _("CSound:"), osutils.find_csound_executables()),
             ("mma", "MMA:", osutils.find_mma_executables(
                 cfg.get_list("app/win32_ignore_drives"))),
@@ -266,11 +299,11 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             for p in bins:
                 liststore.append((p,))
             cbox_entry = gtk.ComboBoxEntry(liststore)
-            cbox_entry.child.set_text(self.get_string("programs/%(bin)s" % locals()))
-            def csound_changed_cb(widget, bin):
-                self.set_string('programs/%s' % bin, widget.child.get_text())
+            cbox_entry.child.set_text(self.get_string("programs/%(binary)s" % locals()))
+            def csound_changed_cb(widget, binary):
+                self.set_string('programs/%s' % binary, widget.child.get_text())
                 widget.warning.props.visible = not bool(
-                    osutils.find_progs((cfg.get_string('programs/%s' % bin),)))
+                    osutils.find_progs((cfg.get_string('programs/%s' % binary),)))
             cbox_entry.warning = gtk.Image()
             cbox_entry.warning.set_tooltip_text(_("Not found. Much of GNU Solfege will run fine without this program. You will get a message when the program is required, and the user manual will explain what you need it for."))
             cbox_entry.warning.set_from_stock(gtk.STOCK_DIALOG_WARNING,
@@ -280,21 +313,14 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                                       sizegroup, True, True)
             category_vbox.pack_start(box)
             cbox_entry.warning.props.no_show_all = True
-            csound_changed_cb(cbox_entry, bin)
-            cbox_entry.connect('changed', csound_changed_cb, bin)
-
-        #######
-        # Gui #
-        #######
-        page_vbox = gu.hig_dlg_vbox()
-        self.g_notebook.append_page(page_vbox, gtk.Label(_("Interface")))
-
-        box, category_vbox = gu.hig_category_vbox(_("Interface"))
-        page_vbox.pack_start(box, False)
+            csound_changed_cb(cbox_entry, binary)
+            cbox_entry.connect('changed', csound_changed_cb, binary)
+    def create_gui_config(self):
+        i_iter, page_vbox = self.new_page_box(None, _("Interface"))
 
         self.g_mainwin_user_resizeable = gu.nCheckButton('gui',
           'mainwin_user_resizeable', _("_Resizeable main window"))
-        category_vbox.pack_start(self.g_mainwin_user_resizeable, False)
+        page_vbox.pack_start(self.g_mainwin_user_resizeable, False)
 
         # Combobox to select language
         hbox = gtk.HBox()
@@ -336,14 +362,15 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                     winlang.win32_put_langenviron(None)
         self.g_language.connect_after('changed', f)
         hbox.pack_start(self.g_language, False)
-        category_vbox.pack_start(hbox)
+        page_vbox.pack_start(hbox, False)
         l = gtk.Label(_("You have to restart the program for the language change to take effect."))
         l.set_alignment(0.0, 0.5)
-        category_vbox.pack_start(l)
-        #### idtone accelerators
-        box, category_vbox = gu.hig_category_vbox(_("Identify tone keyboard accelerators"))
-        page_vbox.pack_start(box, False)
+        page_vbox.pack_start(l, False, False)
 
+        self.create_idtone_accels_config(i_iter)
+        self.create_interval_accels_config(i_iter)
+    def create_idtone_accels_config(self, parent):
+        it, page_vbox = self.new_page_box(parent, _("Identify tone keyboard accelerators"))
         self.g_idtone_accels = gtk.ListStore(gobject.TYPE_STRING,
             gobject.TYPE_STRING)
         notenames = ('c', 'cis', 'd', 'dis', 'e', 'f', 'fis',
@@ -366,7 +393,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                     is_unique = False
                     break
             if not is_unique:
-                gu.dialog_ok(_("The accelerator in use for the tone “%s”. You have to choose another key.") % solfege.mpd.MusicalPitch.new_from_notename(notename).get_user_notename(), parent=self, msgtype=gtk.MESSAGE_ERROR)
+                gu.dialog_ok(_(u"The accelerator in use for the tone “%s”. You have to choose another key.") % solfege.mpd.MusicalPitch.new_from_notename(notename).get_user_notename(), parent=self, msgtype=gtk.MESSAGE_ERROR)
                 return
             it = self.g_idtone_accels.get_iter(path)
             cfg.set_string('idtone/tone_%s_ak' % notenames[int(path)], unichr(accel_key))
@@ -375,39 +402,102 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         renderer.connect('accel-edited', acc_ff)
         column = gtk.TreeViewColumn(_i("keyboard|Key"), renderer, text=1)
         self.g_treeview.append_column(column)
-        category_vbox.pack_start(self.g_treeview)
-        ############
-        # Practise #
-        ############
-        page_vbox = gu.hig_dlg_vbox()
-        self.g_notebook.append_page(page_vbox, gtk.Label(_("Practise")))
-        box, category_vbox = gu.hig_category_vbox(_("Practise"))
-        page_vbox.pack_start(box, False)
-
+        page_vbox.pack_start(self.g_treeview)
+        layouts = {'ascii': (_('ASCII'), u'awsedfujikol'),
+                   'dvorak': (_('Dvorak'), u'a,o.eughctrn'),
+        }
+        hbox = gtk.HBox()
+        page_vbox.pack_start(hbox, False)
+        def set_buttons(widget, layout):
+            v = layouts[layout][1]
+            idx = 0
+            it = self.g_idtone_accels.get_iter_first()
+            while True:
+                self.g_idtone_accels.set_value(it, 1, v[idx])
+                cfg.set_string('idtone/tone_%s_ak' % notenames[idx], v[idx])
+                it = self.g_idtone_accels.iter_next(it)
+                idx += 1
+                if not it:
+                    break
+        for key in layouts:
+            btn = gtk.Button(layouts[key][0])
+            btn.connect('clicked', set_buttons, key)
+            hbox.pack_start(btn)
+    def create_interval_accels_config(self, parent):
+        it, page_vbox = self.new_page_box(parent, _("Interval keyboard accelerators"))
+        self.g_interval_accels = gtk.ListStore(gobject.TYPE_STRING,
+            gobject.TYPE_STRING)
+        intervals = ['minor2', 'major2', 'minor3', 'major3',
+                     'perfect4', 'diminished5', 'perfect5', 'minor6',
+                     'major6', 'minor7', 'major7', 'perfect8',
+                     'minor9', 'major9', 'minor10', 'major10']
+        for interval in intervals:
+            self.g_interval_accels.append((
+                mpd.Interval.new_from_int(intervals.index(interval)).get_name(),
+                cfg.get_string('interval_input/%s' % interval)))
+        self.g_intervals_treeview = gtk.TreeView(self.g_interval_accels)
+        renderer = gtk.CellRendererText()
+        column = gtk.TreeViewColumn(_("Interval"), renderer, text=0)
+        self.g_intervals_treeview.append_column(column)
+        renderer = gtk.CellRendererAccel()
+        renderer.set_property('editable', True)
+        def acc_ff(renderer, path, accel_key, accel_mods, hw_key):
+            is_unique = True
+            for interval in intervals:
+                if (interval != intervals[int(path)]
+                    and cfg.get_string('interval_input/%s' % interval) == unichr(accel_key)):
+                    is_unique = False
+                    break
+            if not is_unique:
+                gu.dialog_ok(_(u"The accelerator in use for “%s”. You have to choose another key.") % mpd.Interval.new_from_int(intervals.index(interval)).get_name(), parent=self, msgtype=gtk.MESSAGE_ERROR)
+                return
+            it = self.g_interval_accels.get_iter(path)
+            cfg.set_string('interval_input/%s' % intervals[int(path)], unichr(accel_key))
+            self.g_interval_accels.set(it, 1, unichr(accel_key))
+            return True
+        renderer.connect('accel-edited', acc_ff)
+        column = gtk.TreeViewColumn(_i("keyboard|Key"), renderer, text=1)
+        self.g_intervals_treeview.append_column(column)
+        page_vbox.pack_start(self.g_intervals_treeview)
+        hbox = gtk.HBox()
+        page_vbox.pack_start(hbox, False)
+        layouts = {'ascii': (_('ASCII'), u'1qaz2wsx3edc4rfv'),
+                   'dvorak': (_('Dvorak'), u"1'a;2,oq3.ej4puk"),
+        }
+        def set_buttons(widget, layout):
+            v = layouts[layout][1]
+            idx = 0
+            it = self.g_interval_accels.get_iter_first()
+            while True:
+                self.g_interval_accels.set_value(it, 1, v[idx])
+                cfg.set_string('interval_input/%s' % intervals[idx], v[idx])
+                it = self.g_interval_accels.iter_next(it)
+                idx += 1
+                if not it:
+                    break
+        for key in layouts:
+            btn = gtk.Button(layouts[key][0])
+            btn.connect('clicked', set_buttons, key)
+            hbox.pack_start(btn)
+    def create_practise_config(self):
+        it, page_vbox = self.new_page_box(None, _("Practise"))
         self.g_picky_on_new_question = gu.nCheckButton('config', 'picky_on_new_question', _("_Not allow new question before the old is solved"))
-        category_vbox.pack_start(self.g_picky_on_new_question, False)
+        page_vbox.pack_start(self.g_picky_on_new_question, False)
 
         self.g_autorepeat_if_wrong = gu.nCheckButton('config', 'auto_repeat_question_if_wrong_answer', _("_Repeat question if the answer was wrong"))
-        category_vbox.pack_start(self.g_autorepeat_if_wrong, False)
+        page_vbox.pack_start(self.g_autorepeat_if_wrong, False)
 
         self.g_expert_mode = gu.nCheckButton('gui',
                 'expert_mode', _("E_xpert mode"))
         self.g_expert_mode.connect('toggled', solfege.app.reset_exercise)
-        category_vbox.pack_start(self.g_expert_mode, False)
-
-
-        #########
-        # sound #
-        #########
+        page_vbox.pack_start(self.g_expert_mode, False)
+    def create_sound_config(self):
         if sys.platform == 'win32':
             self.create_win32_sound_page()
         else:
             self.create_linux_sound_page()
-        ##############
-        # statistics #
-        ##############
-        page_vbox = gu.hig_dlg_vbox()
-        self.g_notebook.append_page(page_vbox, gtk.Label(_("Statistics")))
+    def create_statistics_config(self):
+        it, page_vbox = self.new_page_box(None, _("Statistics"))
         box, category_vbox = gu.hig_category_vbox(_("Statistics from 3.15 and older"))
         page_vbox.pack_start(box, False)
         self.g_old_stat_info = gtk.Label()
@@ -429,7 +519,6 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         category_vbox.pack_start(b, False)
         self.update_statistics_info()
         self.update_old_statistics_info()
-        self.show_all()
     def add_gui_for_external_programs(self, page_vbox):
         box, category_vbox = gu.hig_category_vbox(_("Audio File Players"))
         page_vbox.pack_start(box, False)
@@ -516,7 +605,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                                 widget.child.get_text())
             cmd_combo.connect('changed', _changed, formatid)
             cmd_combo.opts.connect('changed', _opts_changed, formatid)
-            testbutton = gtk.Button(_("Test"))
+            testbutton = gtk.Button(_("_Test").replace("_", ""))
             testbutton.set_tooltip_text(_("This button is clickable only if the binary is found."))
             cmd_combo.testbutton = testbutton
             testbutton.connect('clicked', self.test_XXX_player,
@@ -532,38 +621,53 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             soundcard.play_mediafile(typeid, os.path.join('exercises/standard/lesson-files/share', testfile))
         except osutils.BinaryBaseException, e:
             solfege.win.display_error_message2(e.msg1, e.msg2)
+    def popup_alsa_connection_list(self, widget):
+        connection_list = soundcard.alsa_sequencer.get_connection_list()
+        if connection_list:
+            menu = gtk.Menu()
+            for clientid, portid, clientname, portname, labeltext in connection_list:
+                item = gtk.MenuItem(labeltext)
+                item.set_data('client-port-id', (clientid, portid))
+                menu.append(item)
+                def ff(widget, clientid, portid):
+                    self.g_alsa_device.set_label(widget.get_child().get_text())
+                    self.m_gui_client_port = (clientid, portid)
+                    self.g_alsa_radio.set_active(True)
+                item.connect('activate', ff, clientid, portid)
+            menu.show_all()
+            menu.popup(None, None, None, 1, 0)
     def create_linux_sound_page(self):
-        page_vbox = gu.hig_dlg_vbox()
-        self.g_notebook.append_page(page_vbox, gtk.Label(_("Sound Setup")))
+        it, page_vbox = self.new_page_box(None, _("Sound Setup"))
         #############
         # midi setup
         #############
-        box, category_vbox = gu.hig_category_vbox(_("Sound Setup"))
-        page_vbox.pack_start(box)
 
         self.g_fakesynth_radio = gu.RadioButton(None, _("_No sound"), None)
-        category_vbox.pack_start(self.g_fakesynth_radio, False)
+        page_vbox.pack_start(self.g_fakesynth_radio, False)
 
         ### ALSA
-        hbox = gu.bHBox(category_vbox, False)
+        self.m_gui_client_port = False
+        hbox = gu.bHBox(page_vbox, False)
         self.g_alsa_radio = gu.RadioButton(self.g_fakesynth_radio,
               _("Use ALSA _device"), None)
         hbox.pack_start(self.g_alsa_radio, False)
 
-        self.g_alsa_device = gtk.ComboBox()
-        cell = gtk.CellRendererText()
-        self.g_alsa_device.pack_start(cell, True)
-        self.g_alsa_device.add_attribute(cell, 'text', 0)
+        self.g_alsa_device = gtk.Button()
+        self.g_alsa_device.connect('clicked', self.popup_alsa_connection_list)
         hbox.pack_start(self.g_alsa_device, False)
         if solfege.soundcard.alsa_sequencer:
-            liststore = gtk.ListStore(gobject.TYPE_STRING)
-            v = self.get_list("sound/alsa-client-port")
-            for clientid, portid, clientname, portname in soundcard.alsa_sequencer.get_connection_list():
-                liststore.append(("%i:%i %s %s" % (clientid, portid, clientname, portname),))
-            self.g_alsa_device.set_model(liststore)
-            for idx, v in enumerate(solfege.soundcard.alsa_sequencer.get_connection_list()):
+            connections = solfege.soundcard.alsa_sequencer.get_connection_list()
+            for v in connections:
                 if v[0:2] == self.get_list("sound/alsa-client-port"):
-                    self.g_alsa_device.set_active(idx)
+                    self.g_alsa_device.set_label(v[-1])
+                    self.m_gui_client_port = v[0:2]
+                    break
+            else:
+                if connections:
+                    self.m_gui_client_port = connections[-1][0:2]
+                    self.g_alsa_device.set_label(connections[-1][-1])
+                else:
+                    self.g_alsa_device.set_label("")
         else:
             self.g_alsa_device.set_sensitive(False)
             self.g_alsa_radio.set_sensitive(False)
@@ -571,7 +675,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             label.show()
             hbox.pack_start(label, False)
         ### OSS
-        hbox = gu.bHBox(category_vbox, False)
+        hbox = gu.bHBox(page_vbox, False)
         self.g_device_radio = gu.RadioButton(self.g_fakesynth_radio,
               _("Use OSS _device"), None)
         hbox.pack_start(self.g_device_radio, False)
@@ -588,7 +692,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         hbox.pack_start(self.g_synth_num, False)
 
         ###
-        hbox = gu.bHBox(category_vbox, False)
+        hbox = gu.bHBox(page_vbox, False)
         self.g_midiplayer_radio = gu.RadioButton(self.g_fakesynth_radio,
              _("Use _external MIDI player"), None)
         hbox.pack_start(self.g_midiplayer_radio, False)
@@ -602,28 +706,22 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         else:
             self.g_fakesynth_radio.set_active(True)
 
-        hbox = gtk.HButtonBox()
-        category_vbox.pack_start(hbox)
-        gu.bButton(category_vbox, _("_Test"), self.on_apply_and_play_test_sound)
-
+        gu.bButton(page_vbox, _("_Test"), self.on_apply_and_play_test_sound, False)
     def create_win32_sound_page(self):
-        page_vbox = gu.hig_dlg_vbox()
-        self.g_notebook.append_page(page_vbox, gtk.Label(_("MIDI Setup")))
+        it, page_vbox = self.new_page_box(None, _("Sound Setup"))
         #############
         # midi setup
         #############
-        box, category_vbox = gu.hig_category_vbox(_("Sound Setup"))
-        page_vbox.pack_start(box)
         txt = gtk.Label(_("""Solfege has two ways to play MIDI files. It is recommended to use Windows multimedia output. An external MIDI player can be useful if your soundcard lacks a hardware synth, in which case you have to use a program like timidity to play the music."""))
         txt.set_line_wrap(1)
         txt.set_justify(gtk.JUSTIFY_FILL)
         txt.set_alignment(0.0, 0.0)
-        category_vbox.pack_start(txt, False)
+        page_vbox.pack_start(txt, False)
 
         self.g_fakesynth_radio = gu.RadioButton(None, _("_No sound"), None)
-        category_vbox.pack_start(self.g_fakesynth_radio, False)
+        page_vbox.pack_start(self.g_fakesynth_radio, False)
 
-        hbox = gu.bHBox(category_vbox, False)
+        hbox = gu.bHBox(page_vbox, False)
         self.g_device_radio = gu.RadioButton(self.g_fakesynth_radio,
               _("_Windows multimedia output:"), None)
         self.g_synth = gtk.combo_box_new_text()
@@ -639,7 +737,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         hbox.pack_start(self.g_device_radio, False)
         hbox.pack_start(self.g_synth, False)
 
-        hbox = gu.bHBox(category_vbox, False)
+        hbox = gu.bHBox(page_vbox, False)
         self.g_midiplayer_radio = gu.RadioButton(self.g_fakesynth_radio,
              _("Use _external MIDI player"), None)
         hbox.pack_start(self.g_midiplayer_radio, False)
@@ -651,7 +749,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         else:
             self.g_fakesynth_radio.set_active(True)
 
-        gu.bButton(category_vbox, _("_Test"), self.on_apply_and_play_test_sound)
+        gu.bButton(page_vbox, _("_Test"), self.on_apply_and_play_test_sound, False)
     def set_gui_from_config(self):
         if self.get_string("sound/type") == "fake-synth":
             self.g_fakesynth_radio.set_active(True)
@@ -663,7 +761,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                 p = self.get_list("sound/alsa-client-port")
                 for idx, k in enumerate(solfege.soundcard.alsa_sequencer.get_connection_list()):
                     if p[0] == k[0] and p[1] == k[1]:
-                        self.g_alsa_device.set_active(idx)
+                        self.g_alsa_device.set_label(k[-1])
                         break
             else:
                 self.set_string("sound/type", "fake-synth")
@@ -706,8 +804,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         if soundcard.synth:
             soundcard.synth.close()
         if self.g_midiplayer_radio.get_active():
-            soundcard.initialise_external_midiplayer(
-                  cfg.get_string("sound/midi_player"))
+            soundcard.initialise_external_midiplayer()
             soundcard.synth.error_report_cb = solfege.win.display_error_message
         elif self.g_device_radio.get_active():
             try:
@@ -721,8 +818,15 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                 solfege.app.display_sound_init_error_message(e)
                 return -1
         elif solfege.soundcard.alsa_sequencer and self.g_alsa_radio.get_active():
-            v = solfege.soundcard.alsa_sequencer.get_connection_list()[self.g_alsa_device.get_active()]
-            soundcard.initialise_alsa_sequencer(v[0:2])
+            if self.m_gui_client_port:
+                try:
+                    soundcard.initialise_alsa_sequencer(self.m_gui_client_port)
+                except solfege.soundcard.alsa_sequencer.alsaseq.SequencerError, e:
+                    logging.debug("initialise_alsa_sequencer(%s) failed: %s", self.m_gui_client_port, str(e))
+                    solfege.app.display_sound_init_error_message(e)
+                    return -1
+            else:
+                return -1
         else: # no sound
             assert self.g_fakesynth_radio.get_active()
             soundcard.initialise_using_fake_synth(0)
@@ -735,8 +839,7 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                 self.set_string("sound/type", "sequencer-device")
         elif solfege.soundcard.alsa_sequencer and self.g_alsa_radio.get_active():
             self.set_string("sound/type", "alsa-sequencer")
-            v = solfege.soundcard.alsa_sequencer.get_connection_list()[self.g_alsa_device.get_active()]
-            self.set_list("sound/alsa-client-port", v[0:2])
+            self.set_list("sound/alsa-client-port", self.m_gui_client_port)
         else:
             assert self.g_fakesynth_radio.get_active()
             self.set_string("sound/type", "fake-synth")
