@@ -1,6 +1,6 @@
 # -*- coding: iso-8859-1 -*-
 # GNU Solfege - free ear training software
-# Copyright (C) 2001, 2002, 2003, 2004, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2001, 2002, 2003, 2004, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -96,6 +96,14 @@ class istr(unicode):
     def __init__(self, s):
         self.cval = s
         self.m_added_language = None
+    def __mod__(self, other):
+        """
+        Handle format strings in translated strings:
+        _("%i. inversion") % 2
+        """
+        i=istr(unicode(self) % other)
+        i.cval = self.cval % other
+        return i
     def add_translation(self, lang, s):
         """
         Use this method to add translations that are included directly in
@@ -125,11 +133,15 @@ class istr(unicode):
     new_translated = staticmethod(new_translated)
 
 def dataparser_i18n_func(s):
+    if s == "":
+        s = 'error in your lesson file: empty string cannot be translated'
     retval = istr(_(s))
     retval.cval = s
     return retval
 
 def dataparser_i18n__i_func(s):
+    if s == "":
+        s = 'error in your lesson file: empty string cannot be translated'
     retval = istr(_i(s))
     retval.cval = s
     return retval
