@@ -1,6 +1,6 @@
 # GNU Solfege - free ear training software
 # vim: set fileencoding=utf-8 :
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ from __future__ import division
 
 import os
 import re
+import subprocess
 import sys
 import time
 import traceback
@@ -29,7 +30,9 @@ import gtk
 import pango
 
 from solfege import cfg
+from solfege import osutils
 from solfege import soundcard
+from solfege import utils
 
 import solfege
 
@@ -162,7 +165,7 @@ class nCheckButton(gtk.CheckButton, cfg.ConfigUtils):
         self.set_bool(self.m_name, self.get_active())
         if self.m_callback:
             self.m_callback(_o)
- 
+
 def RadioButton(group, label, callback=None):
     rdb = gtk.RadioButton(group, label)
     if callback:
@@ -170,6 +173,25 @@ def RadioButton(group, label, callback=None):
     rdb.show()
     return rdb
 
+def nComboBox(exname, name, default, popdown_strings):
+    c = gtk.combo_box_new_text()
+    for n in popdown_strings:
+        c.append_text(n)
+    c.m_exname = exname
+    c.m_name = name
+    val = cfg.get_string("%s/%s=X" % (c.m_exname, c.m_name))
+    if val == 'X':
+        cfg.set_int("%s/%s" % (exname, name),
+            popdown_strings.index(default))
+        c.set_active(popdown_strings.index(default))
+    else:
+        c.set_active(cfg.get_int("%s/%s" % (c.m_exname, c.m_name)))
+    def f(combobox):
+        cfg.set_int("%s/%s" % (exname, name), combobox.get_active())
+    c.connect('changed', f)
+    return c
+
+
 class nCombo(gtk.Combo, cfg.ConfigUtils):
     def __init__(self, exname, name, default, popdown_strings):
         """
@@ -225,17 +247,22 @@ class PercussionNameButton(gtk.Button, cfg.ConfigUtils):
     def entry_changed(self, widget, value):
         self.set_int(self.m_name, value + soundcard.first_percussion_int_value)
         self.set_label(soundcard.percussion_names[value])
-
+        t = utils.new_percussion_track()
+        t.note(8, value + soundcard.first_percussion_int_value)
+        t.note(16, value + soundcard.first_percussion_int_value)
+        t.note(16, value + soundcard.first_percussion_int_value)
+        t.note(4, value + soundcard.first_percussion_int_value)
+        soundcard.synth.play_track(t)
 
 class FlashBar(gtk.Frame):
     def __init__(self):
         gtk.Frame.__init__(self)
         self.set_shadow_type(gtk.SHADOW_IN)
-        #FIXME different gtk themes can make these values wrong
-        self.set_size_request(-1, 40)
         self.__stack = []
         self.__label = HarmonicProgressionLabel('')
-        self.add(self.__label)
+        self.__content = gtk.HBox()
+        self.__content.show()
+        self.add(self.__content)
         self.__timeout = None
         # The allocated size
         self.m_sx, self.m_sy = 0, 0
@@ -246,41 +273,57 @@ class FlashBar(gtk.Frame):
         show all the strings in stringlist.
         """
         for s in stringlist:
-            self.__label.set_text(s)
-            x, y =  self.__label.size_request()
-            self.m_sx = max(x, self.m_sx)
-            self.m_sy = max(y, self.m_sy)
-        self.__label.set_text("")
-        self.__label.set_size_request(self.m_sx, self.m_sy)
+            self.display(s)
+        self.empty()
     def delayed_flash(self, milliseconds, msg):
         gobject.timeout_add(milliseconds, lambda: self.flash(msg))
-    def flash(self, txt):
+    def empty(self):
+        for child in self.__content.get_children():
+            child.destroy()
+    def flash(self, txt, **kwargs):
         """Display a message that is automatically removed after some time.
         If we flash a new message before the old flashed message are removed,
         we old flashed message are removed.
         """
         if self.__timeout:
             gobject.source_remove(self.__timeout)
-        self.__label.set_size_request(-1, -1)
-        self.__label.set_text(txt)
-        sx, sy = self.__label.size_request()
-        self.m_sx = max(sx, self.m_sx)
-        self.m_sy = max(sy, self.m_sy)
-        self.__label.set_size_request(self.m_sx, self.m_sy)
+        self.display(txt, **kwargs)
         def f(self=self):
             self.__timeout = None
             if self.__stack:
-                self.__label.set_text(self.__stack[-1])
+                self.display(self.__stack[-1][0], **self.__stack[-1][1])
             else:
-                self.__label.set_text('')
+                self.empty()
         self.__timeout = gobject.timeout_add(2000, f)
-    def push(self, txt):
+    def push(self, txt, **kwargs):
         # stop any flashing before we push
+        self.display(txt, **kwargs)
+        self.__stack.append((txt, kwargs))
+    def display(self, txt, **kwargs):
+        self.empty()
+        r = re.compile("(\{\w+\})") # Unicode??
+        self.set_size_request(-1, -1)
+        for child in r.split(txt):
+            m = r.match(child)
+            if m:
+                varname = child[1:][:-1]
+                from solfege import lessonfilegui
+                w = lessonfilegui.new_labelobject(kwargs[varname])
+                w.show()
+                self.__content.pack_start(w, False)
+            else:
+                if child:
+                    label = gtk.Label(child)
+                    label.set_name("FlashBarLabel")
+                    self.__content.pack_start(label, False)
+                    label.show()
+        sx, sy = self.size_request()
+        self.m_sx = max(sx, self.m_sx)
+        self.m_sy = max(sy, self.m_sy)
+        self.set_size_request(self.m_sx, self.m_sy)
         if self.__timeout:
             gobject.source_remove(self.__timeout)
             self.__timeout = None
-        self.__stack.append(txt)
-        self.__label.set_text(txt)
     def pop(self):
         """If a message is being flashed right now, that flashing is
         not affected, but the message below the flashed message is removed.
@@ -289,19 +332,20 @@ class FlashBar(gtk.Frame):
             self.__stack.pop()
         if not self.__timeout:
             if self.__stack:
-                self.__label.set_text(self.__stack[-1])
+                self.display(self.__stack[-1][0],
+                           **self.__stack[-1][1])
             else:
-                self.__label.set_text('')
+                self.empty()
     def clear(self):
         self.__stack = []
         if not self.__timeout:
-            self.__label.set_text('')
-    def set(self, txt):
+            self.empty()
+    def set(self, txt, **kwargs):
         """
         Empty the stack of messages and display the string in txt.
         """
-        self.__stack = [txt]
-        self.__label.set_text(txt)
+        self.__stack = [(txt, kwargs)]
+        self.display(txt, **kwargs)
 
 class hig(object):
     SPACE_SMALL= 6
@@ -505,7 +549,7 @@ def dialog_delete(text, parent, secondary_text=None):
     m.destroy()
     return ret == gtk.RESPONSE_ACCEPT
     return ret
-        
+
 class NewLineBox(gtk.VBox):
     def __init__(self):
         gtk.VBox.__init__(self)
@@ -818,34 +862,14 @@ class EditorDialogBase(object):
         return ret
 
 
-class LogWindow(gtk.Window):
-    def __init__(self, parent):
-        gtk.Window.__init__(self)
-        self.set_transient_for(parent)
-        self.set_destroy_with_parent(True)
-        self.set_modal(True)
-        vbox = gtk.VBox()
-        vbox.set_spacing(4)
-        self.add(vbox)
-        self.set_default_size(630, 400)
+class LogWidget(gtk.ScrolledWindow):
+    def __init__(self):
+        gtk.ScrolledWindow.__init__(self)
         self.g_textbuffer = gtk.TextBuffer()
         self.g_textbuffer.create_tag('h1', weight=pango.WEIGHT_BOLD,
             size=16*pango.SCALE)
         self.g_textview = gtk.TextView(buffer=self.g_textbuffer)
-        self.g_scrolledwindow = gtk.ScrolledWindow()
-        vbox.pack_start(self.g_scrolledwindow)
-        self.g_scrolledwindow.add(self.g_textview)
-        vbox.pack_start(gtk.HSeparator(), False, False)
-        bbox = gtk.HButtonBox()
-        bbox.set_layout(gtk.BUTTONBOX_END)
-        vbox.pack_start(bbox, False, False)
-        self.g_close_button = gtk.Button(stock="gtk-close")
-        self.g_close_button.set_sensitive(False)
-        self.g_close_button.connect('clicked', lambda w: self.destroy())
-        bbox.pack_start(self.g_close_button)
-        self.show_all()
-    def run_finished(self):
-        self.g_close_button.set_sensitive(True)
+        self.add(self.g_textview)
     def write(self, s, tag=None):
         if tag:
             self.g_textbuffer.insert_with_tags_by_name(
@@ -858,6 +882,55 @@ class LogWindow(gtk.Window):
         # This is needed to make the window update:
         while gtk.events_pending():
             gtk.main_iteration()
+    def popen(self, *args, **kwargs):
+        """
+        Return the Popen objects .returncode
+        """
+        self.write("$ %s\n" % " ".join(args[0]))
+        p = osutils.Popen(
+            stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT,
+            *args, **kwargs)
+        while 1:
+            p.poll()
+            # returncode != None means that the process has finished
+            if p.returncode != None:
+                break
+            while 1:
+                s = p.stdout.readline()
+                if not s:
+                    break
+                self.write(s)
+            time.sleep(1)
+        p.wait()
+        return p.returncode
+
+
+class LogWindow(gtk.Window):
+    def __init__(self, parent):
+        gtk.Window.__init__(self)
+        self.set_transient_for(parent)
+        self.set_destroy_with_parent(True)
+        self.set_modal(True)
+        vbox = gtk.VBox()
+        vbox.set_spacing(4)
+        self.add(vbox)
+        self.set_default_size(630, 400)
+        self.g_logwidget = LogWidget()
+        vbox.pack_start(self.g_logwidget)
+        vbox.pack_start(gtk.HSeparator(), False, False)
+        bbox = gtk.HButtonBox()
+        bbox.set_layout(gtk.BUTTONBOX_END)
+        vbox.pack_start(bbox, False, False)
+        self.g_close_button = gtk.Button(stock="gtk-close")
+        self.g_close_button.set_sensitive(False)
+        self.g_close_button.connect('clicked', lambda w: self.destroy())
+        bbox.pack_start(self.g_close_button)
+        self.show_all()
+        self.write = self.g_logwidget.write
+        self.popen = self.g_logwidget.popen
+    def run_finished(self):
+        self.g_close_button.set_sensitive(True)
 
 
 class ClickableLabel(gtk.LinkButton):
